/*!
 * @file  lince5m_utils.h
 * @brief Sensor-specific computations in a form that is easy to share.
 *
 * Copyright 2014 (c) Fastec Imaging Corporation as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of Fastec
 * Imaging Corporation. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement with
 * Fastec Imaging Corporation.
 */

#ifndef _LINCE5M_UTILS_H_
#define _LINCE5M_UTILS_H_
#ifdef __cplusplus
extern "C" {
#endif

/*! GUI support function for getting the maximum allowed frame rate */
uint32_t lince5m_max_fps_setting( uint16_t startx, uint16_t starty,
                                  uint16_t width, uint16_t height, uint8_t bit_mode, uint32_t sensopts );

/*! GUI support function to get maximum size for a bit mode */
void lince5m_max_roi_setting( uint8_t bit_mode, uint16_t *rtn_width, uint16_t *rtn_height );

/*! Lower level functions to compute maximum frame rate, used by Camera Server */
int lince5m_user_to_sensor( unsigned factor, uint16_t startx, uint16_t starty,
                            uint16_t width, uint16_t height, uint8_t bit_mode,
                            uint16_t *rtn_startx, uint16_t *rtn_starty,
                            uint16_t *rtn_width, uint16_t *rtn_height);
int lince5m_get_readout_dims( unsigned factor, uint16_t sensor_startx, uint16_t sensor_width,
                              uint16_t *rtn_startx, uint16_t *rtn_width );
uint32_t lince5m_get_max_frame_rate( uint32_t readout_width, uint32_t readout_height, uint32_t sensopts );
uint32_t lince5m_get_max_frame_rate_trow( uint32_t readout_width, uint32_t readout_height,
        uint32_t sensopts, uint32_t t_row_cycles );

/*! Get the t_row configuration to use for this readout width. */
uint32_t lince5m_get_minimum_trow( uint32_t readout_width, uint32_t sensor_opts );
/*! Get the t_row configuration to use for this ROI. */
uint32_t lince5m_get_trow_from_user( uint32_t x, uint32_t y, uint32_t width, uint32_t height,
                                     uint8_t bit_mode, uint32_t sensor_opts, uint32_t frame_rate );
/*! Get the binning/subsampling factor from the sensor options */
unsigned lince5m_binsubs_factor( uint32_t sensor_opts );
/*! Get the adjustment to the real sensor offset of the user ROI within the capturable area. */
void lince5m_get_ct_offset( uint8_t bit_mode, uint32_t sensor_opts, uint16_t *rtn_ofs_x, uint16_t * rtn_ofs_y );

#ifdef __cplusplus
}
#endif
#endif /* _LINCE5M_UTILS_H_ */
